RandomYield = {
  yield = nil,
  fruitTypesYield = {}
}

-- Function to save the yield value to an XML file
function RandomYield:saveYield()
  local xmlFile = createXMLFile("RandomYieldSave", getUserProfileAppPath() .. "RandomYield.xml", "RandomYield")
  setXMLFloat(xmlFile, "RandomYield.yield", self.yield)
  saveXMLFile(xmlFile)
  delete(xmlFile)
end

-- Function to load the yield value from an XML file
function RandomYield:loadYield()
  local xmlFile = loadXMLFile("RandomYieldSave", getUserProfileAppPath() .. "RandomYield.xml")
  if xmlFile ~= nil then
    self.yield = getXMLFloat(xmlFile, "RandomYield.yield")
    delete(xmlFile)
  end

  -- If no saved yield is found, generate a new one
  if self.yield == nil then
    self.yield = math.random() * 1.7 + 0.5
    self:saveYield()
  end
end

function RandomYield:randomizeYields()
  for _, fruitType in ipairs(g_currentMission.fruitTypeManager.fruitTypes) do
    local soilData = g_farmlandManager:getFarmlandAtWorldPosition(0, 0) -- Adjust position as needed
    local soilMultiplier = 1

    if soilData ~= nil and soilData.fertilization ~= nil and soilData.pH ~= nil then
      soilMultiplier = soilData.fertilization * soilData.pH -- Adjust calculation based on soil properties
    end

    self.fruitTypesYield[fruitType.index] = fruitType.literPerSqm
    fruitType.literPerSqm = self.fruitTypesYield[fruitType.index] * self.yield * soilMultiplier
  end
end

function RandomYield:loadMap(name)
  if g_currentMission:getIsServer() then
    self:loadYield()
    self:randomizeYields()  -- Randomize yields based on the loaded yield value
  end
  g_messageCenter:subscribe(MessageType.PERIOD_CHANGED, self.onPeriodChanged, self)
end

function RandomYield:onPeriodChanged(data)
  -- Recalculate yield each month
  self.yield = math.random() * 1.7 + 0.5
  if self.yield > 1.5 then
    self.yield = g_currentMission.fruitTypeManager.defaultLiterPerSqm
  end
  self:saveYield()
  
  if g_currentMission:getIsServer() then
    self:randomizeYields()
  end
end

function RandomYield:loadCustomMap(name)
  if g_currentMission:getIsServer() then
    self:loadYield()
    self:randomizeYields()  -- Randomize yields based on the loaded yield value
  end
  g_messageCenter:subscribe(MessageType.PERIOD_CHANGED, self.onPeriodChanged, self)
end

function RandomYield:load()
  addModEventListener(self)
end

function RandomYield:delete()
  g_messageCenter:unsubscribeAll(self)
end

addModEventListener(RandomYield)